/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package account

import "testing"

func TestGenAccountAddress(t *testing.T) {
	key := "-----BEGIN PRIVATE KEY-----\nMIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQDOQWZp1JDnO6+u\nNu6Ox8Ka3IRyKg8qrAHLyzj6nXNEoP4S3meocuLmbT47DdiPXpNKXhyNOkVytuRs\nXt4etoxX4zFh1j+8oHQkbKjNZvGBgUVCxLcmmPYhFEiz9lt1pteLxHVpoPlScUer\nMBltuDWwB6NFFFKeqrPWwH/vS7FCVJc/orU9AD9pNEbigXh5LGz/E6an+DL4Eb7V\nEzOhl2nwaX4Q6R7b8bt0iRLUMCLpGbhj2+UWgWzQ727F6dZHSCqxgPl5oP7ahP34\n34XVtVbdGz06BzlFJGKHrNoqZkBiN7P6aA0+ndILg7WdbaymRnAjx7cXxxDSA82V\nNVyPgnhDAgMBAAECggEAAzIpO5eTHTokCT+Jp8W5orurA7faH0+mAXnnKp7488XI\n2hL/KTGKiYaE0dZvlh0hjCP0Lk4KWHI0DOfVakVq3EvFbp4qw6n60ruqS/B96oNN\nxvCUY/E30yBk1gEJvsN0kJJdPWNEqb3enezzX+tcTZt4sK/Vbeb4PWojMVqLZxsw\nU72edKlM2dJPEGDWbAhrXjz3wu0oUosvKf2WtMgPqDzCI54hQu7KoGLsrdK75X4l\nzjr4ILfCqxAPUaAtDuDVlP2UJpS0cEFZ5uN8i2bjriRVuW2PaJ8tA1QTDuxwOdDa\njsQKQZwHNQA8QUQnpxBKtitcCoEvtbHG6rX34wbbvQKBgQDmzMKYV7TGmhdlS8Wh\n2aKPE7Kg732+/AVj4dAyBVG3uGELL0ByfSDkofWRqMuSf1Ub8KuepKii402SEBML\nHsDKZT3JEpaszoyHS1EWsYEelu1wEKABmwMqkTtIg7CbsduQrnWQCTGvmjOXlGZl\n566fc//uMdht4e6cz3SHUah0fQKBgQDkxpVxp3e2CW6Wi0nRHBy8WfaDalmeSQ+U\ngmL48isdqT5tfiP6IAgIUFuGBdpcrVFUYdnX2t8erQoBmFhy6YhlAXn7zQxVFPkN\n33+0HesXnskfyCZYQ8JFkxjcZaul4qHflGYxq1R77ZvuxHtiq/PCdAQ74vnM8Pfr\nczjN7qH7vwKBgFyiUnhSu3v63OkLQId1kolanpd5iD7faEpul+F7wRC8MDcNABDz\nqAAcATQ1S3/BG6fbMQa7j/PQx5GYYgxUi4ywldveWhuBy2WCFUnGdMySRzpzccUz\nMlIjeGfic0AKYtXvL8Bf8FFK0upD5dWA59WX+WDhILz5sYu+uEaXv8UBAoGBAJNu\n8wJitYBIBc/ZBgJQBqjPpVE9qXGbjgHcIWJj1qtv1X6EqzTXjbWl68WsYfQV8fLa\neKRmNiIzy94JSv8LR04orG88zrTKJlLtdQQjv8ziFKV5vFRnnnxt9riagQsPydgn\nb9CUpIaVM0elsgp93EK1t8DMXsXXiT5ga/wyzItLAoGBALt5qnXjx++cp6wJ2x1m\npAXKh2I/DVSj/d9Tl+WRuJF2dFWK8qpz49LdhPJUnvfEYSWW5eR/i76JTLk2crMu\n2PzhMJ9XcCdCkSMcZE6q3ZRhmoHKXDmdW8bUA0CMHujwuggL5/hAyNboNA32qp4L\nU92htBiXcDcV8u7sYxq8k/Dq\n-----END PRIVATE KEY-----"
	addCM, err := GenAccountAddress(addressTypeCM, []byte(key))
	if err != nil {
		t.Fatalf("failed to gen CM address,err: %v", err)
	}

	t.Logf("success to gen CM address: %s", addCM)

	addEVM, err := GenAccountAddress(addressTypeEVM, []byte(key))
	if err != nil {
		t.Fatalf("failed to gen EVM address,err: %v", err)
	}

	t.Logf("success to gen EVM address: %s", addEVM)

	addZXL, err := GenAccountAddress(addressTypeZXL, []byte(key))
	if err != nil {
		t.Fatalf("failed to gen ZXL address,err: %v", err)
	}

	t.Logf("success to gen ZXL address: %s", addZXL)
}
