/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package code defines some response code
package code

// RespCode 服务端返回码
type RespCode int

// Success 服务端返回码，200 表示成功，其他表示失败
const (
	Success RespCode = 200
)

// 32000 - 32999 表示 key-service grpc 错误码
const (
	ErrSm4EncryptKey RespCode = iota + 32000
	ErrSm4EncryptCert
	ErrGenAccountAddress
	ErrGetPublicKey
	ErrInsertKeyInfo
	ErrCreateKvs
	ErrSendCallContactReq
	ErrExecuteCallContact
	ErrExecuteTx
	ErrConvertRegisterId
	ErrUpdateKeyInfo
	ErrCountKeyInfo
	ErrListKeyInfo
	ErrQueryByChainInfoId
	ErrNotFoundKey
	ErrBase64DecodeKeyCipher
	ErrSm4DecryptKey
	ErrBase64DecodeCertCipher
	ErrSm4DecryptCert
	ErrParseKeyFromPem
	ErrDecodePem
	ErrParseCertFromPem
	ErrUnknownKeyMode
	ErrJsonUnmarshalPayload
	ErrCreateSigner
	ErrSign
	ErrNewMember
	ErrListByUserId
	ErrEmptyAccountInfo
	ErrDeleteByKeyId
	ErrQueryByKeyId
	ErrGenerateKey
	ErrEncryptMsg
	ErrDecryptMsg
	InvalidAlgo
	ErrInsertKey
	ErrParamInput
	ErrNotImpl
	ErrDidNotFindSk
	ErrNotSupportedMultiKeys
	ErrChainServiceUnReachable
	ErrRegisterIdaUser
	ErrPKEncodeType
	ErrEmptyTxResponse
	ErrInitCrypto
	ErrSignTxPayload
	ErrUnmarshalKmsKey
)

// 返回码对应具体的信息
var errMsg = map[RespCode]string{
	Success:                    "success",
	ErrSm4EncryptKey:           "failed to sm4 encrypt key",
	ErrSm4EncryptCert:          "failed to sm4 encrypt cert",
	ErrGenAccountAddress:       "failed to gen account address",
	ErrGetPublicKey:            "failed to get public key",
	ErrInsertKeyInfo:           "failed to insert table keyInfo",
	ErrCreateKvs:               "failed to create key -value pairs",
	ErrSendCallContactReq:      "failed to send callContract req",
	ErrExecuteCallContact:      "failed to execute callContract",
	ErrExecuteTx:               "failed to execute tx",
	ErrConvertRegisterId:       "failed to convert register id",
	ErrUpdateKeyInfo:           "failed to update table keyInfo",
	ErrCountKeyInfo:            "failed to count table keyInfo",
	ErrListKeyInfo:             "failed to list table keyInfo",
	ErrQueryByChainInfoId:      "failed to query key by chainInfoId",
	ErrNotFoundKey:             "not found key",
	ErrBase64DecodeKeyCipher:   "failed to base64 decode key cipher",
	ErrSm4DecryptKey:           "failed to sm4 decrypt key",
	ErrBase64DecodeCertCipher:  "failed to base64 decode cert cipher",
	ErrSm4DecryptCert:          "failed to sm4 decrypt cert",
	ErrParseKeyFromPem:         "failed to parse key from pem",
	ErrDecodePem:               "failed to decode pem data",
	ErrParseCertFromPem:        "failed to parse cert from pem data",
	ErrUnknownKeyMode:          "unknown key mode",
	ErrJsonUnmarshalPayload:    "failed to json unmarshal tx payload",
	ErrCreateSigner:            "failed to create signer",
	ErrSign:                    "failed to sign",
	ErrNewMember:               "failed to new signer member",
	ErrListByUserId:            "failed to list key info by user id",
	ErrEmptyAccountInfo:        "empty account info",
	ErrDeleteByKeyId:           "failed to delete key info by id",
	ErrQueryByKeyId:            "failed to query key info by id",
	ErrNotSupportedMultiKeys:   "not supported to import multi keys",
	ErrChainServiceUnReachable: "chain-service is un-reachable",
	ErrGenerateKey:             "failed to generate key",
	ErrEncryptMsg:              "failed to encrypt message",
	ErrDecryptMsg:              "failed to decrypt message",
	InvalidAlgo:                "invalid algorithm",
	ErrInsertKey:               "failed to insert key",
	ErrParamInput:              "param input error",
	ErrNotImpl:                 "not implemented",
	ErrDidNotFindSk:            "did not find sk",
	ErrRegisterIdaUser:         "failed to register ida user",
	ErrPKEncodeType:            "PK encoding type is not hex",
	ErrEmptyTxResponse:         "got empty tx response",
	ErrInitCrypto:              "init crypto error",
	ErrSignTxPayload:           "failed to sign tx payload",
	ErrUnmarshalKmsKey:         "failed to unmarshal kms key",
}

func (rc RespCode) String() string {
	return errMsg[rc]
}
