/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

// Package config defines the server config
package config

import (
	"github.com/zeromicro/go-zero/gateway"
	"github.com/zeromicro/go-zero/zrpc"
)

// Config contain all config items for server
type Config struct {
	zrpc.RpcServerConf
	GrpcConf             GrpcConf
	DBConf               DBConf
	SymmetricEncryptConf SymmetricEncryptConf
	AddressType          string
	ChainServiceConf     ChainServiceConf
	Gateway              gateway.GatewayConf
	CryptoConfig         CryptoConfig
	SDFConf              SDFConf
	Pkcs11Conf           Pkcs11Conf
	KmsConf              KmsConf
}

// KmsConf kms 配置项
type KmsConf struct {
	Enabled    bool
	DefaultUrl string
}

const (
	// AlgoAes aes 加密算法
	AlgoAes = "AES"

	// AlgoSm4 sm4 加密算法
	AlgoSm4 = "SM4"
)

// SymmetricEncryptConf 用于加密私钥的对称加密配置
type SymmetricEncryptConf struct {

	// 加密算法: AES, SM4
	Algo string

	// 加密密钥
	Key string
}

// DBConf 数据库连接配置
type DBConf struct {
	Type string
	DNS  string
}

// GrpcConf contain all config items for grpc server initiation
type GrpcConf struct {
	CaCertFile     string
	ServerCertFile string
	ServerKeyFile  string
	MaxRecvMsgSize int
	MaxSendMsgSize int
}

// ChainServiceConf contain all config items for connecting chain-service
type ChainServiceConf struct {
	ClientCertFile string
	ClientKeyFile  string
	CaCertFile     string
	DNS            string
	Endpoint       string
}

// CryptoConfig - crypto config
type CryptoConfig struct {
	Soft bool
	HSM  string
}

// SDFConf - sdf config
type SDFConf struct {
	LibPath          string
	SessionCacheSize int
}

// Pkcs11Conf - pkcs11 config
type Pkcs11Conf struct {
	Enabled          bool
	Type             string
	Label            string
	Library          string
	Password         string
	SessionCacheSize int
	Hash             string
}
