package csp

import (
	"errors"

	internal_config "chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/config"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/csp/pkcs11"
	_const "chainweaver.org.cn/chainweaver/mira/mira-common/crypto/const"
	"chainweaver.org.cn/chainweaver/mira/mira-common/crypto/hsmimpl/sdf"
)

var (

	// Csp 全局共用 csp 对象
	Csp CSP
)

// CSP - crypto service provider
type CSP interface {
	// Enc returns the encrypted data against the specified algorithm
	Enc(algo _const.Algorithm, key string, keyPwd []byte, plainText string, mode string) (string, error)

	// Dec returns the decrypted data against the specified algorithm
	Dec(algo _const.Algorithm, key string, keyPwd []byte, cipherText string, mode string) (string, error)
}

// InitCrypto returns a new crypto instance. support soft and csp
func InitCrypto(cfg *internal_config.Config) (CSP, error) {
	var err error
	// if crypto is soft encrypt
	if cfg.CryptoConfig.Soft {
		Csp, err = NewSoft()
		if err != nil {
			return nil, err
		}
		// if crypto is sdf encrypt
	} else if cfg.CryptoConfig.HSM == "sdf" {
		Csp, err = sdf.NewSDF(cfg.SDFConf.LibPath)
		if err != nil {
			return nil, err
		}
		//  if crypto is pkcs11 encrypt
	} else if cfg.CryptoConfig.HSM == "pkcs11" {
		Csp, err = pkcs11.NewPKCS11(&cfg.Pkcs11Conf)
		if err != nil {
			return nil, err
		}
	} else {
		return nil, errors.New("not support hsm type")
	}
	return Csp, nil
}
