package pkcs11

import (
	"testing"

	"chainmaker.org/chainmaker/common/v2/crypto/pkcs11"
	internal_config "chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/config"
	_const "chainweaver.org.cn/chainweaver/mira/mira-common/crypto/const"
	"chainweaver.org.cn/chainweaver/mira/mira-common/utils"
)

// 测试NewPKCS11
func TestNewPKCS11(t *testing.T) {
	utils.SkipTest(t)
	type args struct {
		cfg *internal_config.Pkcs11Conf
	}
	tests := []struct {
		name string
		args args
	}{
		{
			name: "success",
			args: args{
				cfg: &internal_config.Pkcs11Conf{
					Enabled:          true,
					Type:             "pkcs11",
					Label:            "HSM",
					Library:          "/usr/lib/softhsm/libsofthsm2.so",
					Password:         "11111111",
					SessionCacheSize: 10,
					Hash:             "SHA256",
				},
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := NewPKCS11(tt.args.cfg)
			if err != nil {
				t.Errorf("NewPKCS11() error = %v", err)
				return
			}
			if got == nil {
				t.Errorf("NewPKCS11() got = %v", got)
			}
		})
	}
}

// 测试Dec
func TestPKCS11_Dec(t *testing.T) {
	utils.SkipTest(t)
	type fields struct {
		P11Handle *pkcs11.P11Handle
	}
	type args struct {
		algo       _const.Algorithm
		key        string
		keyPwd     []byte
		cipherText string
		mode       string
	}
	tests := []struct {
		name   string
		fields fields
		args   args
	}{
		{
			name: "success",
			args: args{
				algo:   _const.SM2,
				key:    "test",
				keyPwd: []byte("11"),
				cipherText: "04340ee40e3f6847f561a1c243ff8d6c35595434e46e3038e5e9b" +
					"f68ff52a5f06b32260e4bd04089864ca4ab7598b9310b47bbfab90aa802" +
					"a2a042585e92b246e1f5f0dd68658953ac04b09747e83bd83fb40" +
					"b85607de06e2f110cf35bb7dc3e6e7ecf84c99d126f",
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			pkcs11 := PKCS11{
				P11Handle: tt.fields.P11Handle,
			}
			got, err := pkcs11.Dec(tt.args.algo, tt.args.key, tt.args.keyPwd, tt.args.cipherText, tt.args.mode)
			if err != nil {
				t.Errorf("Dec PKCS11() error = %v", err)
				return
			}
			if got == "" {
				t.Errorf("Dec PKCS11() got = %v", got)
			}
		})
	}
}

// 测试Enc
func TestPKCS11_Enc(t *testing.T) {
	utils.SkipTest(t)
	type fields struct {
		P11Handle *pkcs11.P11Handle
	}
	type args struct {
		algo      _const.Algorithm
		key       string
		keyPwd    []byte
		plainText string
		mode      string
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    string
		wantErr bool
	}{
		{
			name: "success",
			args: args{
				algo:      _const.SM2,
				key:       "test",
				keyPwd:    []byte("11"),
				plainText: "test",
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			pkcs11 := PKCS11{
				P11Handle: tt.fields.P11Handle,
			}
			got, err := pkcs11.Enc(tt.args.algo, tt.args.key, tt.args.keyPwd, tt.args.plainText, tt.args.mode)
			if (err != nil) != tt.wantErr {
				t.Errorf("Enc() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got != tt.want {
				t.Errorf("Enc() got = %v, want %v", got, tt.want)
			}
		})
	}
}
