package csp

import (
	"errors"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	_const "chainweaver.org.cn/chainweaver/mira/mira-common/crypto/const"
)

// SoftImpl implements soft CSP
type SoftImpl struct {
}

// NewSoft 构造soft CSP
func NewSoft() (*SoftImpl, error) {
	return &SoftImpl{}, nil
}

// Enc implements soft CSP.Enc
// @param algo
// @param key
// @param keyPwd
// @param plainText
// @param mode
func (s SoftImpl) Enc(algo _const.Algorithm, key string, keyPwd []byte, plainText string, mode string) (string, error) {
	// 判断算法类型
	switch algo {
	// 如果是SM4
	case _const.SM4:
		// 使用SM4加密
		return svc.SM4Encrypt(key, plainText)
	//	如果是SM2
	case _const.SM2:
		// 使用SM2加密
		return svc.SM2Encrypt(key, plainText)
	//	默认返回错误
	default:
		return "", errors.New("only support [SM2 | SM4] now")
	}
}

// Dec implements soft CSP.Dec
// @param algo
// @param key
// @param keyPwd
// @param cipherText
// @param mode
func (s SoftImpl) Dec(algo _const.Algorithm, key string, keyPwd []byte, cipherText string,
	mode string) (string, error) {
	// 判断算法类型
	switch algo {
	// 如果是SM4
	case _const.SM4:
		// 使用SM4解密
		return svc.SM4Decrypt(key, cipherText)
	//	如果是SM2
	case _const.SM2:
		// 使用SM2解密
		return svc.SM2Decrypt(key, cipherText)
	//	默认返回错误
	default:
		return "", errors.New("only support [SM2 | SM4] now")
	}
}
