package csp

import (
	"testing"

	_const "chainweaver.org.cn/chainweaver/mira/mira-common/crypto/const"
)

// TestNewSoft tests the NewSoft function.
func TestNewSoft(t *testing.T) {
	tests := []struct {
		name    string
		wantErr bool
	}{
		{
			name: "success",
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			got, err := NewSoft()
			if (err != nil) != tt.wantErr {
				t.Errorf("NewSoft() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got == nil {
				t.Errorf("NewSoft() got = %v", got)
			}
		})
	}
}

// TestSoftImpl_Dec tests the Dec function.
func TestSoftImpl_Dec(t *testing.T) {
	type args struct {
		algo       _const.Algorithm
		key        string
		keyPwd     []byte
		cipherText string
		mode       string
	}
	tests := []struct {
		name    string
		args    args
		want    string
		wantErr bool
	}{
		{
			name: "success",
			args: args{
				algo: _const.SM2,
				key:  "ebe1d9ad102d9465f94e86e8479eae811b83bf6204d778bf4670718fa188d236",
				cipherText: "04a9b30d523f581ead025b2a536445bbd83ad1ef544186ff113fd" +
					"84eda0208f7fe0c2793f9589643f98e436234832eca4e42054c52d92990c655d5c09" +
					"4769dc225f59c8d4760b5cbd0eb88c739a3a71c8095400d7f5323adfeb8e64328dcecb2" +
					"e1193e4ebd7ebe25",
			},
			want: "cxhtest",
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			s := SoftImpl{}
			got, err := s.Dec(tt.args.algo, tt.args.key, tt.args.keyPwd, tt.args.cipherText, tt.args.mode)
			if (err != nil) != tt.wantErr {
				t.Errorf("Dec() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got != tt.want {
				t.Errorf("Dec() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// TestSoftImpl_Enc tests the Enc function.
func TestSoftImpl_Enc(t *testing.T) {
	type args struct {
		algo      _const.Algorithm
		key       string
		keyPwd    []byte
		plainText string
		mode      string
	}
	tests := []struct {
		name    string
		args    args
		wantErr bool
	}{
		{
			name: "success",
			args: args{
				algo:      _const.SM2,
				key:       "04c4fc8b0e428c7157c72f374a6a0306f2146ed30793ae6be6ab1d2f1b01467a6c84af7c5c105d50735e0ed560ae34826a8a2e5105a13490b5f2f4ca0efd0410fc",
				plainText: "cxhtest",
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			s := SoftImpl{}
			got, err := s.Enc(tt.args.algo, tt.args.key, tt.args.keyPwd, tt.args.plainText, tt.args.mode)
			if (err != nil) != tt.wantErr {
				t.Errorf("Enc() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got == "" {
				t.Errorf("Enc() got = %v", got)
			}
		})
	}
}
