/*
 * @Author: chenxuanhui xuanhuichen@tencent.com
 * @Date: 2024-05-28 11:38:00
 * @LastEditors: chenxuanhui xuanhuichen@tencent.com
 * @LastEditTime: 2024-05-29 18:11:48
 * @FilePath: /key-service/internal/logic/createkeylogic_test.go
 * @Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 */
package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"chainmaker.org/chainmaker/common/v2/crypto/sym/sm4"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"chainweaver.org.cn/chainweaver/mira/mira-common/types"
	"github.com/golang/mock/gomock"
	"github.com/zeromicro/go-zero/core/logx"
)

// createKeySVC 用于创建密钥结构
func createKeySVC(t *testing.T) *svc.ServiceContext {
	t.Helper()
	service := &svc.ServiceContext{
		SymmetricKey: &sm4.SM4Key{
			Key: []byte(sm4Key),
		},
		AddressType: addressType,
	}
	return service
}

// TestCreateKeyLogic_CreateKey 测试CreateKey
func TestCreateKeyLogic_CreateKey(t *testing.T) {
	ctl := gomock.NewController(t)
	defer ctl.Finish()
	key := &types.SecretKey{}
	mockDBAccessInterface := dbaccess.NewMockDBAccessInterface(ctl)
	mockDBAccessInterface.EXPECT().AddSecretKeyReturnKey(gomock.Any()).Return(key, nil)

	dbaccess.DBAccessService = mockDBAccessInterface
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		in *keypb.KeyCreateRequest
	}
	tests := []struct {
		name   string
		fields fields
		args   args
		want   *keypb.KeyCreateResponse
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.KeyCreateRequest{
					AlgoType: keypb.AlgoType_SM2,
					KeySize:  12,
				},
			},
			want: &keypb.KeyCreateResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &CreateKeyLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, _ := l.CreateKey(tt.args.in)
			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}

// TestCreateKeyLogic_errorResponse 测试errorResponse
func TestCreateKeyLogic_errorResponse(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		code code.RespCode
		err  error
	}

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				code: code.ErrDidNotFindSk,
				err:  errors.New(code.ErrDidNotFindSk.String()),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &CreateKeyLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.errorResponse(tt.args.code, tt.args.err)
			if err == nil {
				t.Errorf("errorResponse() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// TestNewCreateKeyLogic 测试NewCreateKeyLogic
func TestNewCreateKeyLogic(t *testing.T) {
	type args struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
	}
	tests := []struct {
		name string
		args args
		want *CreateKeyLogic
	}{
		{
			name: "success",
			args: args{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := NewCreateKeyLogic(tt.args.ctx, tt.args.svcCtx); got == nil {
				t.Errorf("NewCreateKeyLogic() = %v, want %v", got, tt.want)
			}
		})
	}
}
