/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"github.com/zeromicro/go-zero/core/logx"
)

// DeleteKeyAndCertLogic defines the object to delete key and cert
type DeleteKeyAndCertLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewDeleteKeyAndCertLogic create the object to delete key and cert
func NewDeleteKeyAndCertLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DeleteKeyAndCertLogic {
	return &DeleteKeyAndCertLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// DeleteKeyAndCert 删除私钥和证书
func (l *DeleteKeyAndCertLogic) DeleteKeyAndCert(
	in *keypb.DeleteKeyAndCertRequest) (*keypb.DeleteKeyAndCertResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId": in.RequestId,
		"keyId":     in.KeyId,
	}

	// 从本地数据库中移除链账户信息
	err := l.svcCtx.KeyInfoModel.DeleteByKeyId(in.KeyId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrDeleteByKeyId.String())
		return &keypb.DeleteKeyAndCertResponse{
			Code: int32(code.ErrDeleteByKeyId),
			Msg:  err.Error(),
		}, nil
	}

	// 成功返回
	return &keypb.DeleteKeyAndCertResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
	}, nil
}
