package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"github.com/zeromicro/go-zero/core/logx"
)

// 测试EncryptWithPkListLogic
func TestEncryptWithPkListLogic_EncryptWithPkList(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		in *keypb.KeyEncryptWithPkListRequest
	}
	pkList := []string{
		"046e6f33c95c54386215d8a96eee116d92e51f7a1d5b402994c6b51529723f60eea6d09560b7fdbcd6043b075a51b483c61972a76dd8fbaa8b09dc398c146a2b3c",
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.KeyEncryptWithPkListResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createEncryptSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.KeyEncryptWithPkListRequest{
					AlgoType:   keypb.AlgoType_SM2,
					PlainText:  "test plain",
					PubKeyList: pkList,
				},
			},
			want: &keypb.KeyEncryptWithPkListResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &EncryptWithPkListLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.EncryptWithPkList(tt.args.in)
			if (err != nil) != tt.wantErr {
				t.Errorf("EncryptWithPkList() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试EncryptWithPkListLogic_errorResponse
func TestEncryptWithPkListLogic_errorResponse(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		code code.RespCode
		err  error
	}

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				code: code.ErrDidNotFindSk,
				err:  errors.New(code.ErrDidNotFindSk.String()),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &EncryptWithPkListLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.errorResponse(tt.args.code, tt.args.err)
			if err == nil {
				t.Errorf("errorResponse() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试NewEncryptWithPkListLogic
func TestNewEncryptWithPkListLogic(t *testing.T) {
	type args struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
	}
	tests := []struct {
		name string
		args args
		want *CreateKeyLogic
	}{
		{
			name: "success",
			args: args{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := NewEncryptWithPkListLogic(tt.args.ctx, tt.args.svcCtx); got == nil {
				t.Errorf("NewCreateKeyLogic() = %v, want %v", got, tt.want)
			}
		})
	}
}
