package logic

import (
	"context"
	"errors"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"github.com/zeromicro/go-zero/core/logx"
)

// 测试EncWithDeKLogic
func TestEncWithDeKLogic_EncWithDeK(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		in *keypb.DataEnvelopeEncryptRequest
	}
	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.DataEnvelopeEncryptResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createEncryptSVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				in: &keypb.DataEnvelopeEncryptRequest{
					DataEnvelopAlgoType: keypb.DataEnvelopAlgoType_SM2SM4DataEnvelope,
					PlainText:           "cxhtest",
					PubKey:              "04c4fc8b0e428c7157c72f374a6a0306f2146ed30793ae6be6ab1d2f1b01467a6c84af7c5c105d50735e0ed560ae34826a8a2e5105a13490b5f2f4ca0efd0410fc",
				},
			},
			want: &keypb.DataEnvelopeEncryptResponse{
				Code: int32(code.Success),
				Msg:  code.Success.String(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &EncWithDeKLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.EncWithDeK(tt.args.in)
			if (err != nil) != tt.wantErr {
				t.Errorf("EncWithDeK() error = %v, wantErr %v", err, tt.wantErr)
				return
			}
			if got.Code != tt.want.Code || got.Msg != tt.want.Msg {
				t.Errorf("AccountConfig got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试EncWithDeKLogic_errorResponse
func TestEncWithDeKLogic_errorResponse(t *testing.T) {
	type fields struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
		Logger logx.Logger
	}
	type args struct {
		code code.RespCode
		err  error
	}

	tests := []struct {
		name    string
		fields  fields
		args    args
		want    *keypb.GetSkResponse
		wantErr bool
	}{
		{
			name: "success",
			fields: fields{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
				Logger: logx.WithContext(context.Background()),
			},
			args: args{
				code: code.ErrDidNotFindSk,
				err:  errors.New(code.ErrDidNotFindSk.String()),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			l := &EncWithDeKLogic{
				ctx:    tt.fields.ctx,
				svcCtx: tt.fields.svcCtx,
				Logger: tt.fields.Logger,
			}
			got, err := l.errorResponse(tt.args.code, tt.args.err)
			if err == nil {
				t.Errorf("errorResponse() got = %v, want %v", got, tt.want)
			}
		})
	}
}

// 测试NewEncWithDeKLogic
func TestNewEncWithDeKLogic(t *testing.T) {
	type args struct {
		ctx    context.Context
		svcCtx *svc.ServiceContext
	}
	tests := []struct {
		name string
		args args
		want *CreateKeyLogic
	}{
		{
			name: "success",
			args: args{
				ctx:    context.Background(),
				svcCtx: createKeySVC(t),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			if got := NewEncWithDeKLogic(tt.args.ctx, tt.args.svcCtx); got == nil {
				t.Errorf("NewCreateKeyLogic() = %v, want %v", got, tt.want)
			}
		})
	}
}
