/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package logic

import (
	"context"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/model/table"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/util"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"

	"github.com/zeromicro/go-zero/core/logx"
)

// GetAccountDetailLogic defines the object to get account detail
type GetAccountDetailLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewGetAccountDetailLogic create the object to get account detail
func NewGetAccountDetailLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetAccountDetailLogic {
	return &GetAccountDetailLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// GetAccountDetail 链账户详情
func (l *GetAccountDetailLogic) GetAccountDetail(
	in *keypb.GetAccountDetailRequest) (*keypb.GetAccountDetailResponse, error) {

	// 日志通用信息
	fields := map[string]interface{}{
		"requestId": in.RequestId,
		"keyId":     in.KeyId,
	}

	// 从本地数据库查询链账户信息
	info, _, err := l.svcCtx.KeyInfoModel.QueryByKeyId(in.KeyId)
	if err != nil {
		fields["err"] = err

		// 转成结构化日志信息
		l.Logger.WithFields(util.ConvertToLogFields(fields)...).Error(code.ErrQueryByKeyId.String())
		return &keypb.GetAccountDetailResponse{
			Code: int32(code.ErrQueryByKeyId),
			Msg:  err.Error(),
		}, nil
	}

	// 成功返回
	return &keypb.GetAccountDetailResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),

		// 转成 response data 结构
		Data: util.ConvertToAccountData([]table.KeyInfo{*info}, nil)[0],
	}, nil
}
