package logic

import (
	"context"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"github.com/zeromicro/go-zero/core/logx"
)

// GetSkFromKeyIdLogic struct.
type GetSkFromKeyIdLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewGetSkFromKeyIdLogic creates a new GetSkFromKeyIdLogic.
func NewGetSkFromKeyIdLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetSkFromKeyIdLogic {
	return &GetSkFromKeyIdLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// GetSkFromKeyId 根据 keyId 获取私钥
func (l *GetSkFromKeyIdLogic) GetSkFromKeyId(in *keypb.GetSkFromKeyIdRequest) (*keypb.GetSkResponse, error) {
	// 从数据库里获取key
	key, err := dbaccess.DBAccessService.GetSecretKeyById(in.KeyId)
	if err != nil {
		return l.errorResponse(code.ErrDidNotFindSk, err)
	}

	// 成功获取秘钥
	return l.successResponse(key.PrivateKey), nil
}

// errorResponse returns the error response.
func (l *GetSkFromKeyIdLogic) errorResponse(code code.RespCode, err error) (*keypb.GetSkResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.GetSkResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *GetSkFromKeyIdLogic) successResponse(sk string) *keypb.GetSkResponse {
	return &keypb.GetSkResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		Sk:   sk,
	}
}
