package logic

import (
	"context"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/code"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/svc"
	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"chainweaver.org.cn/chainweaver/mira/mira-common/dbaccess"
	"github.com/zeromicro/go-zero/core/logx"
)

// GetSkFromPkLogic struct.
type GetSkFromPkLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

// NewGetSkFromPkLogic creates a new GetSkFromPkLogic.
func NewGetSkFromPkLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetSkFromPkLogic {
	return &GetSkFromPkLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// GetSkFromPk 根据公钥获取私钥
func (l *GetSkFromPkLogic) GetSkFromPk(in *keypb.GetSkRequest) (*keypb.GetSkResponse, error) {
	// 从数据库里获取key
	key, err := dbaccess.DBAccessService.GetSecretKeyByPubKey(in.PubKey)
	if err != nil {
		return l.errorResponse(code.ErrDidNotFindSk, err)
	}

	// 成功获取公钥
	return l.successResponse(key.PrivateKey), nil
}

// errorResponse returns the error response.
func (l *GetSkFromPkLogic) errorResponse(code code.RespCode, err error) (*keypb.GetSkResponse, error) {
	msg := code.String()
	if err != nil {
		msg = err.Error()
	}

	return &keypb.GetSkResponse{
		Code: int32(code),
		Msg:  msg,
	}, err
}

// successResponse returns the success response.
func (l *GetSkFromPkLogic) successResponse(sk string) *keypb.GetSkResponse {
	return &keypb.GetSkResponse{
		Code: int32(code.Success),
		Msg:  code.Success.String(),
		Sk:   sk,
	}
}
