/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package model

import (
	"database/sql"
	"fmt"
	"testing"
	"time"

	"github.com/DATA-DOG/go-sqlmock"
	"github.com/stretchr/testify/assert"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
)

//func initLocalDB() *KeyInfoModel {
//	c := config.Config{
//		MysqlConf: config.MysqlConf{
//			DNS:                       "root:root123456@tcp(localhost:3306)/keyservice?charset=utf8&parseTime=true&loc=Asia%2FShanghai",
//			MaxLifetime:               500,
//			MaxIdleConns:              10,
//			MaxOpenConns:              10,
//			DefaultStringSize:         256,
//			DisableDatetimePrecision:  true,
//			DontSupportRenameIndex:    true,
//			DontSupportRenameColumn:   true,
//			SkipInitializeWithVersion: false,
//		},
//	}
//	return NewKeyInfoModel(c)
//}

func newTestDB() (KeyInfoModel, *sql.DB, sqlmock.Sqlmock) {
	db, mock, err := sqlmock.New()
	if err != nil {
		panic(fmt.Errorf("an error '%s' was not expected when opening a stub database connection", err))
	}

	gormDB, err := gorm.Open(mysql.New(mysql.Config{
		Conn:                      db,
		SkipInitializeWithVersion: true,
	}))
	if err != nil {
		panic(fmt.Errorf("failed to open gorm db,err: %v", err))
	}

	return KeyInfoModel{gormDB}, db, mock
}

func TestKeyInfoModel_Insert(t *testing.T) {
	testDB, db, mock := newTestDB()
	defer db.Close()
	mock.ExpectBegin()
	mock.ExpectExec("INSERT INTO `key_info`").
		WithArgs(sqlmock.AnyArg(), sqlmock.AnyArg(), sqlmock.AnyArg(), "123", "a", "testChain", "KEY_MODE_PK", "dfdssdfgsdf", "server.key",
			"application/x-iwork-keynote-sffkey", "", "", "", "0xsadsd", "", "", "", 0, 1, "", "", "").
		WillReturnResult(sqlmock.NewResult(1, 1))
	mock.ExpectCommit()

	err := testDB.Insert("123", "a", "testChain", "KEY_MODE_PK", "dfdssdfgsdf", "server.key",
		"application/x-iwork-keynote-sffkey", "", "", "", "0xsadsd", "", "", 1)
	if err != nil {
		t.Fatalf("error was not expected while inserting key: %s", err)
	}

	// we make sure that all expectations were met
	if err := mock.ExpectationsWereMet(); err != nil {
		t.Errorf("there were unfulfilled expectations: %s", err)
	}
}

//func TestKeyInfoModel_Insert2(t *testing.T) {
//	err := initLocalDB().Insert("123", "a", "testChain", "KEY_MODE_PK", "dfdssdfgsdf", "server.key", "application/x-iwork-keynote-sffkey", "", "", "", "0xsadsd")
//	if err != nil {
//		t.Fatalf("failed to insert key: %s", err)
//	}
//
//	t.Logf("success to insert key")
//}

//func TestKeyInfoModel_Update(t *testing.T) {
//	//err := initLocalDB().Update(1, 1, "testChain2", "00123083GHVJJ", "审计方", map[uint32]string{2: "dad", 3: "qwe"})
//	//if err != nil {
//	//	t.Fatalf("failed to update key: %s", err)
//	//}
//	//
//	//t.Logf("success to update key")
//
//	testDB, db, mock := newTestDB()
//	defer db.Close()
//	mock.ExpectBegin()
//
//	// gorm 自动生成的时间精确到 ms 单位，所以这里为了匹配测试，手动传入转换之后的时间（这两者还是有可能出现毫秒级别的误差，暂时没看到 go-sqlmock 中解决方式）
//	mock.ExpectExec("UPDATE `key_info`").
//		WithArgs("testChain2", "00123083GHVJJ", "审计方", 1, time.UnixMilli(time.Now().UnixMilli()), 1).
//		WillReturnResult(sqlmock.NewResult(1, 1))
//	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
//		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
//	mock.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)
//	rows2 := sqlmock.NewRows([]string{"id", "key_id", "user_id", "user_name", "created_at", "updated_at"}).
//		AddRow(1, 1, 1, "asd", time.Now(), time.Now())
//	mock.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnRows(rows2)
//	mock.ExpectExec("INSERT INTO `auth_user`").
//		WithArgs(1, 1, "jack").
//		WillReturnResult(sqlmock.NewResult(1, 1))
//	mock.ExpectCommit()
//
//	err := testDB.Update(1, 1, "testChain2", "00123083GHVJJ", "审计方", map[uint32]string{1: "jack"})
//	if err != nil {
//		t.Fatalf("error was not expected while updating key: %s", err)
//	}
//
//	// we make sure that all expectations were met
//	if err := mock.ExpectationsWereMet(); err != nil {
//		t.Errorf("there were unfulfilled expectations: %s", err)
//	}
//}

func TestKeyInfoModel_Count(t *testing.T) {
	//count, err := initLocalDB().Count()
	//if err != nil {
	//	t.Fatalf("failed to count key: %s", err)
	//}
	//
	//t.Logf("success to count key: %d", count)

	testDB, db, mock := newTestDB()
	defer db.Close()
	rows := sqlmock.NewRows([]string{"count"}).AddRow(1)
	mock.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows)
	c, err := testDB.Count(0)
	assert.Nil(t, err)
	assert.EqualValues(t, 1, c)
}

func TestKeyInfoModel_List(t *testing.T) {
	//list, auths, err := initLocalDB().List(1, 0)
	//if err != nil {
	//	t.Fatalf("failed to list key: %s", err)
	//}
	//
	//t.Logf("success to list key: %#v - %#v", list, auths)

	testDB, db, mock := newTestDB()
	defer db.Close()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
	mock.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)
	//rows2 := sqlmock.NewRows([]string{"id", "key_id", "user_id", "user_name", "created_at", "updated_at"}).
	//	AddRow(1, 1, 1, "asd", time.Now(), time.Now())
	//mock.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnRows(rows2)
	_, _, err := testDB.List(10, 0, 0)
	assert.Nil(t, err)
	//assert.EqualValues(t, 1, len(authUsers))
}

func TestKeyInfoModel_QueryByChainIdAndUserId(t *testing.T) {
	//ki, err := initLocalDB().QueryByChainIdAndUserId("123", "a", 1)
	//if err != nil {
	//	t.Fatalf("failed to QueryByChainIdAndUserId: %s", err)
	//}
	//
	//t.Logf("success to QueryByChainIdAndUserId: %v", ki)

	testDB, db, mock := newTestDB()
	defer db.Close()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
	mock.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)
	ki, err := testDB.QueryByChainIdAndUserId("chain1", "123", 1)
	assert.Nil(t, err)
	assert.EqualValues(t, "chain1", ki.ChainId)
}

func TestKeyInfoModel_DeleteByKeyId(t *testing.T) {
	//err := initLocalDB().DeleteByKeyId(1)
	//if err != nil {
	//	t.Fatalf("failed to DeleteByKeyId: %s", err)
	//}
	//
	//t.Logf("success to DeleteByKeyId")

	testDB, db, mock := newTestDB()
	defer db.Close()
	mock.ExpectBegin()
	//mock.ExpectExec("DELETE FROM `auth_user`").
	//	WithArgs(1).
	//	WillReturnResult(sqlmock.NewResult(1, 1))
	mock.ExpectExec("UPDATE `key_info` SET").
		WithArgs(sqlmock.AnyArg(), 1).
		WillReturnResult(sqlmock.NewResult(1, 1))
	mock.ExpectCommit()

	err := testDB.DeleteByKeyId(1)
	if err != nil {
		t.Fatalf("error was not expected while deleting key: %s", err)
	}

	// we make sure that all expectations were met
	if err := mock.ExpectationsWereMet(); err != nil {
		t.Errorf("there were unfulfilled expectations: %s", err)
	}
}

func TestKeyInfoModel_QueryByKeyId(t *testing.T) {
	//keyInfo, authUsers, err := initLocalDB().QueryByKeyId(1)
	//if err != nil {
	//	t.Fatalf("failed to QueryByKeyId: %s", err)
	//}
	//
	//t.Logf("success to QueryByKeyId: %#v - %#v", keyInfo, authUsers)

	testDB, db, mock := newTestDB()
	defer db.Close()
	rows1 := sqlmock.NewRows([]string{"id", "chain_id", "sub_chain_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, "chain1", "123", "asd", time.Now(), time.Now())
	mock.ExpectQuery("SELECT (.+) FROM `key_info`").WillReturnRows(rows1)
	//rows2 := sqlmock.NewRows([]string{"id", "key_id", "user_id", "user_name", "created_at", "updated_at"}).
	//	AddRow(1, 1, 1, "asd", time.Now(), time.Now())
	//mock.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnRows(rows2)
	_, _, err := testDB.QueryByKeyId(1)
	assert.Nil(t, err)
}

func TestKeyInfoModel_QueryAuthUserId(t *testing.T) {
	//id, err := initLocalDB().QueryAuthUserId(1, 1)
	//if err != nil {
	//	t.Fatalf("failed to QueryAuthUserId: %s", err)
	//}
	//
	//t.Logf("success to QueryAuthUserId: %v", id)
	//
	//id, err = initLocalDB().QueryAuthUserId(1, 2)
	//if err != nil {
	//	t.Fatalf("failed to QueryAuthUserId: %s", err)
	//}
	//
	//t.Logf("success to QueryAuthUserId: %v", id)

	testDB, db, mock := newTestDB()
	defer db.Close()
	rows2 := sqlmock.NewRows([]string{"id", "key_id", "user_id", "user_name", "created_at", "updated_at"}).
		AddRow(1, 1, 1, "asd", time.Now(), time.Now())
	mock.ExpectQuery("SELECT (.+) FROM `auth_user`").WillReturnRows(rows2)
	id, err := testDB.QueryAuthUserId(1, 1)
	assert.Nil(t, err)
	assert.EqualValues(t, 1, id)
}

func TestKeyInfoModel_DeleteAuthUsers(t *testing.T) {
	//err := initLocalDB().DeleteAuthUsers(1, []uint32{4})
	//if err != nil {
	//	t.Fatalf("failed to DeleteAuthUsers: %s", err)
	//}
	//
	//t.Logf("success to DeleteAuthUsers")

	testDB, db, mock := newTestDB()
	defer db.Close()
	mock.ExpectBegin()
	mock.ExpectExec("DELETE FROM `auth_user`").
		WithArgs(1, 1).
		WillReturnResult(sqlmock.NewResult(1, 1))
	mock.ExpectCommit()

	err := testDB.DeleteAuthUsers(1, []uint32{1})
	if err != nil {
		t.Fatalf("error was not expected while deleting auth users: %s", err)
	}

	// we make sure that all expectations were met
	if err := mock.ExpectationsWereMet(); err != nil {
		t.Errorf("there were unfulfilled expectations: %s", err)
	}
}
