/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package table

import "gorm.io/gorm"

// KeyInfo 私钥信息表
// nolint:lll
type KeyInfo struct {

	// 使用 gorm.Model，表记录的删除是软删除，deleted_at is null 表示未被删除过，即可用的链账户。
	// 不过 gorm 框架自动对更新、删除、查询操作过滤软删记录，所以不需要手动在 sql 中加过滤软删的条件。
	gorm.Model
	ChainId                 string `gorm:"column:chain_id;type:varchar(64);comment:区块链id" json:"chain_id"`
	SubChainId              string `gorm:"column:sub_chain_id;type:varchar(64);comment:子链id" json:"sub_chain_id"`
	ChainName               string `gorm:"column:chain_name;type:varchar(64);comment:区块链名称" json:"chain_name"`
	KeyMode                 string `gorm:"column:key_mode;comment:私钥模式" json:"key_mode"`
	KeyCipher               string `gorm:"column:key_cipher;type:blob;comment:私钥密文" json:"key_cipher"`
	KeyFileName             string `gorm:"column:key_file_name;type:varchar(64);comment:私钥文件名称" json:"key_file_name"`
	KeyFileType             string `gorm:"column:key_file_type;type:varchar(64);comment:私钥文件类型" json:"key_file_type"`
	CertCipher              string `gorm:"column:cert_cipher;type:blob;comment:证书密文" json:"cert_cipher"`
	CertFileName            string `gorm:"column:cert_file_name;type:varchar(64);comment:证书文件名称" json:"cert_file_name"`
	CertFileType            string `gorm:"column:cert_file_type;type:varchar(64);comment:证书文件类型" json:"cert_file_type"`
	AccountAddress          string `gorm:"column:account_address;type:varchar(64);comment:链账户地址" json:"account_address"`
	AccountName             string `gorm:"column:account_name;type:varchar(64);comment:链账户身份" json:"account_name"`
	AccountSocialCreditCode string `gorm:"column:account_social_credit_code;type:varchar(64);comment:社会信用代码" json:"account_social_credit_code"`
	AccountType             string `gorm:"column:account_type;type:varchar(64);comment:账户类型" json:"account_type"`
	RegisterId              uint   `gorm:"column:register_id;comment:链上注册Id" json:"register_id"`
	ChainInfoId             uint   `gorm:"column:chain_info_id;comment:链信息Id" json:"chain_info_id"`
	Alias                   string `gorm:"column:alias;type:varchar(64);comment:链账户别名" json:"alias"`
	Did                     string `gorm:"column:did;comment:链账户did" json:"did"`
	PublicKey               string `gorm:"column:public_key;type:blob;comment:外接kms会保存公钥pem格式内容" json:"public_key"`
}

// TableName 表名称
func (m *KeyInfo) TableName() string {
	return "key_info"
}
