package svc

import (
	"fmt"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/pb/keypb"
	"github.com/stretchr/testify/assert"
)

var (
	plainText = "cxhtest"
	pubKeyHex = "04c4fc8b0e428c7157c72f374a6a0306f2146ed30793ae6be6ab1d2f1b014" +
		"67a6c84af7c5c105d50735e0ed560ae34826a8a2e5105a13490b5f2f4ca0efd0410fc"
)

// TestSM2KeyPairCreate 测试SM2KeyPairCreate
func TestSM2Decrypt(t *testing.T) {
	t.Log("TestSM2Decrypt")
	key, err := SM2KeyPairCreate()
	if err != nil {
		t.Fatalf("failed to SM2KeyPairCreate: %s", err)
	}
	cipherText, err := SM2Encrypt(key.PublicKey, plainText)
	fmt.Println(cipherText)
	if err != nil {
		t.Fatalf("failed to SM2Encrypt: %s", err)
	}
	plainTextResult, err := SM2Decrypt(key.PrivateKey, cipherText)
	if err != nil {
		return
	}
	fmt.Println(plainTextResult)
	assert.Equal(t, plainText, plainTextResult)
}

// TestSM4EncryptAndDecrypt 测试SM4EncryptAndDecrypt
func TestSM4EncryptAndDecrypt(t *testing.T) {
	plain := "hello world"
	cipher, _ := SM4Encrypt("3d6767b9e9ce5d1e", plain)
	plainResult, _ := SM4Decrypt("3d6767b9e9ce5d1e", cipher)
	assert.Equal(t, plain, plainResult)
}

// TestEncWithDeK 测试EncWithDeK
func TestEncWithDeK(t *testing.T) {
	got, got1, err := EncWithDeK(pubKeyHex, plainText)
	if err != nil {
		fmt.Println(err)
		return
	}
	if got == "" || got1 == "" {
		t.Errorf("EncWithDeK error = %v", got)
	}
}

// TestEncWithDekList 测试EncWithDekList
func TestEncWithDekList(t *testing.T) {
	pkList := []string{
		pubKeyHex,
		"04a8ef0effe3bb0506f62f1888ce564877b9aee2a53bb81378a97b24f0ef1d" +
			"83d17d25f2c87657f5ddd4ad98883252248e8c3a6645af070c499ec06d79f991b4b7",
	}
	got, got1, err := EncWithDekList(pkList, plainText)
	if err != nil {
		fmt.Println(err)
		return
	}
	if got == "" || got1 == nil {
		t.Errorf("EncWithDekList error = %v", got)
	}
}

// TestDecByKeK 测试DecByKeK
func TestDecByKeK(t *testing.T) {
	sk := "ebe1d9ad102d9465f94e86e8479eae811b83bf6204d778bf4670718fa188d236"
	cipherText := "8061f82b7a90716af5ef9ca4c081aade"
	kek := "041fcf556069ee5942804f55b6dcdeb8028484ffeee7aac416b2cd6520ad30e74" +
		"2da28f7b794fe019b5bf3c45491bb9cc72a3daf5e8b41102685bfdc50e39ce013495b6af" +
		"01255e5e1bd3c2596eacbf1cfe88f3d0847dd2f780d56a4ecdf15ea5598f7e67ce48bb1d967b4a479f710672a"
	got, err := DecByKeK(sk, cipherText, kek)
	if err != nil {
		fmt.Println(err)
		return
	}
	if got == "" {
		t.Errorf("DecByKeK error = %v", got)
	}
}

// 测试GenSymKey
func TestGenSymKey(t *testing.T) {
	algoTypeSM2 := keypb.AlgoType_SM2
	algoTypeAes := keypb.AlgoType_AES
	keySize := int32(16)
	sm2key, err := GenSymKey(algoTypeSM2, keySize)
	if err != nil {
		fmt.Println(err)
		return
	}
	if sm2key == nil {
		t.Errorf("GenSymKey error = %v", sm2key)
	}
	aesKey, err := GenSymKey(algoTypeAes, keySize)
	if err != nil {
		fmt.Println(err)
		return
	}
	if aesKey == nil {
		t.Errorf("GenSymKey error = %v", aesKey)
	}
}
