/*
Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.

SPDX-License-Identifier: Apache-2.0
*/

package util

import (
	"bytes"
	"testing"

	"chainweaver.org.cn/chainweaver/ida/key-service/v2/internal/model/table"

	"chainmaker.org/chainmaker/pb-go/v2/accesscontrol"
	"gorm.io/gorm"
)

func TestConvertToLogxFields(t *testing.T) {
	m := map[string]interface{}{
		"a": 123,
	}

	logFields := ConvertToLogFields(m)
	if len(logFields) != 1 {
		t.Errorf("wanted length 3,got length %d", len(logFields))
	}

	if logFields[0].Key != "a" && logFields[0].Value != 123 {
		t.Errorf("wanted kv a:123,got kv %s:%d", logFields[0].Key, logFields[0].Value)
	}
}

func TestConvertToAccountData(t *testing.T) {
	infos := []table.KeyInfo{
		{
			Model: gorm.Model{
				ID: 1,
			},
			ChainId:    "123",
			SubChainId: "a",
		},
		{
			Model: gorm.Model{
				ID: 2,
			},
			ChainId:    "456",
			SubChainId: "b",
		},
	}

	authUsers := []map[uint32]string{
		{
			1: "jack",
			2: "lucy",
		},
		{
			1: "jack",
			2: "lucy",
		},
	}

	d := ConvertToAccountData(infos, authUsers)
	if len(d) != 2 {
		t.Errorf("wanted length 2,got length %d", len(d))
	}
}

func TestConvertToSignResponseMember(t *testing.T) {
	m := &accesscontrol.Member{
		OrgId:      "zx",
		MemberType: accesscontrol.MemberType_CERT,
		MemberInfo: []byte("sdffdfds"),
	}

	res := ConvertToSignResponseMember(m)
	if res == nil || res.OrgId != "zx" && res.MemberType != 0 && !bytes.Equal(res.MemberInfo, []byte("sdffdfds")) {
		t.Error("failed to convert")
	}
}

func TestIsAvailablePk(t *testing.T) {

	IsAvailablePk("dsafsf")
}
