# KeyService 加解密接口文档

[toc]

## 1. 公私钥生成 

### 接口描述

- **接口名称**  /v1/CreateKey
- **接口说明**：生成公私钥，同时公私钥会存储在mysql中
- **请求方式**：POST

### 输入参数

| **参数名称** | 必选 | **类型** | **描述**                                            |
| ------------ | ---- | -------- | --------------------------------------------------- |
| algoType     | 是   | String   | 生成公私钥使用的密码学算法，提供：SM2，SM4类型      |
| keySize      | 否   | Integer  | 对称加密的长度，可选：16、24、32。SM4为16位，可不传 |

### 输出参数

| **参数名称** | **类型** | **描述**                                    |
| ------------ | -------- | ------------------------------------------- |
| keyId        | String   | 公私钥存储在Mysql中的id，用于查询对应的私钥 |
| pubKey       | String   | 公钥                                        |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/CreateKey \
  -H 'Content-Type: application/json' \
  -d '{
    "algoType": "SM2",
    "keySize": 0
}'
```

### 示例

（1）输入

```json
{
    "algoType": "SM2",
  	"keySize":0
}
```

（2）输出

```json
{
    "keyId": "202403111715290372", 		        "pubKey":"04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}
```

## 2. 加密

### 接口描述

- **接口名称**  /v1/Encrypt
- **接口说明**：加密数据
- **请求方式**：POST

### 输入参数

| **参数名称** | **必选** | **类型** | **描述**                                    |
| ------------ | -------- | -------- | ------------------------------------------- |
| plainText    | 是       | String   | 明文信息                                    |
| algoType     | 是       | String   | 加密使用的密码学算法,提供：SM2，SM4         |
| pubKey       | 是       | String   | 公钥，pubkey，如果是对称密钥，pubKey就是key |
| extraParam   | 否       | String   | 可选参数，例如序列化类型等                  |

### 输出参数

| **参数名称** | **类型** | **描述** |
| ------------ | -------- | -------- |
| cipherText   | String   | 密文信息 |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/Encrypt \
  -H 'Content-Type: application/json' \
  -d '{
    "algoType": "SM2",
    "plainText": "cxhtest",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}'
```

### 示例

（1）输入

```json
{
    "algoType": "SM2",
    "plainText": "cxhtest",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}
```

（2）输出

```json
{
    "ciperText": "encrypted data"
}
```

## 3. 解密

### 接口描述

- **接口名称**  /v1/Decrypt
- **接口说明**：解密数据
- **请求方式**：POST

### 输入参数

| **参数名称** | **必选** | **类型** | **描述**                                                     |
| ------------ | -------- | -------- | :----------------------------------------------------------- |
| cipherText   | 是       | String   | 密文信息                                                     |
| algoType     | 否       | String   | 如果在数据库中存储，可以为空，解密时会根据pk查类型。加密使用的密码学算法,可选：SM2，SM4 |
| pubKey       | 是       | String   | 公钥，软件使用，如果是对称密钥，pubKey就是key                |
| keyId        | 否       | String   | 硬件使用，和pubKey二选一                                     |

### 输出参数

| **参数名称** | **类型** | **描述**         |
| ------------ | -------- | ---------------- |
| plainText    | String   | 解密后的明文信息 |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/Decrypt \
  -H 'Content-Type: application/json' \
  -d '{
    "algoType": "SM2",
    "cipherText": "04340ee40e3f6847f561a1c243ff8d6c35595434e46e3038e5e9bf68ff52a5f06b32260e4bd04089864ca4ab7598b9310b47bbfab90aa802a2a042585e92b246e1f5f0dd68658953ac04b09747e83bd83fb40b85607de06e2f110cf35bb7dc3e6e7ecf84c99d126f",
    "keyId": "",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}'
```

### 示例

（1）输入

```json
{
    "algoType": "SM2",
    "cipherText": "04340ee40e3f6847f561a1c243ff8d6c35595434e46e3038e5e9bf68ff52a5f06b32260e4bd04089864ca4ab7598b9310b47bbfab90aa802a2a042585e92b246e1f5f0dd68658953ac04b09747e83bd83fb40b85607de06e2f110cf35bb7dc3e6e7ecf84c99d126f",
    "keyId": "",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}
```

（2）输出

```json
{
    "plainText": "text example"
}
```

## 4. 使用公钥列表加密

### 接口描述

- **接口名称**: /v1/EncryptWithPKList
- **接口说明**: 使用公钥列表加密数据
- **请求方式**: POST

### 输入参数

| **参数名称** | **必选** | **类型**          | **描述**                       |
| ------------ | -------- | ----------------- | ------------------------------ |
| plainText    | 是       | String            | 明文信息                       |
| algoType     | 是       | String            | 加密使用的密码学算法,提供：SM2 |
| pubKeyList   | 是       | array of [String] | 公钥列表，pubkey列表           |
| extraParam   | 否       | String            | 预留参数，暂未使用             |

### 输出参数

| **参数名称**   | **类型** | **描述**     |
| -------------- | -------- | ------------ |
| cipherTextList | String   | 密文信息列表 |

### 输入示例

```
curl -X POST \
  http://127.0.0.1:8081/v1/EncryptWithPkList \
  -H 'Content-Type: application/json' \
  -d '{
    "algoType": "SM2",
    "extraParam": "",
    "plainText": "cxhtest",
    "pubKeyList": [ "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2", "040fd782e8cad8ce9563ea179156c7bc374f428b940c29e62a3a095dc5a2da4793480fe7998c45ba03bfb682e89aeff2c7b9b6703c45e85640b3434bf5ab21911c"
    ]
}'
```

### 示例

（1）输入

```json
{
    "algoType": "SM2",
    "extraParam": "",
    "plainText": "cxhtest",
    "pubKeyList": [ "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2", "040fd782e8cad8ce9563ea179156c7bc374f428b940c29e62a3a095dc5a2da4793480fe7998c45ba03bfb682e89aeff2c7b9b6703c45e85640b3434bf5ab21911c"
    ]
}
```

（2）输出

```json
{
    "cipherTextList": [ "04d475530961a7a6300a5fee7369d2249e3aea336d7b502c73bc6a0cba167613075e6662819c12515924afbaa27607b81b083cfe0dd644075c2b9937bb802fc2b3245fc8a4da1db491a05dfcc28ae7aeb400f1b067853304d3d27797fcbeae827fdf09809dfcc65e", "0464f55d25865699a5b0294aad382b0f80093c56bc88c157d3a9eae1c3bcd223bc90d24bfaa05e163754d2394135ddd0f690acce3abdc198bf9fc8f3a3f89411f0ec0f6de59273f7ef10fb47f0f1d9c34e54559a774331f7f03d6704df8d72e129681be3ba447dab"
    ]
}
```

## 5. 数字信封加密

### 接口描述

- **接口名称**: /v1/EncWithDeK
- **接口说明**: 使用数字信封加密数据
- **请求方式**: POST

### 输入参数

| **参数名称**        | **必选** | **类型** | **描述**                                                     |
| ------------------- | -------- | -------- | ------------------------------------------------------------ |
| plainText           | 是       | String   | 明文信息                                                     |
| dataEnvelopAlgoType | 是       | String   | 加密使用的密码学算法,可选：SM2SM4DataEnvelope和Sm2CpAbeDataEnvelope(暂未实现) |
| pubKey              | 是       | String   | 公钥                                                         |
| extraParam          | 否       | String   | 预留参数，暂未使用                                           |

### 输出参数

| **参数名称** | **类型** | **描述** |
| ------------ | -------- | -------- |
| cipherText   | String   | 密文信息 |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/EncWithDeK \
  -H 'Content-Type: application/json' \
  -d '{
    "dataEnvelopAlgoType": "SM2SM4DataEnvelope",
    "plainText": "cxhtest",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}'
```

### 示例

（1）输入

```json
{
    "dataEnvelopAlgoType": "SM2SM4DataEnvelope",
    "plainText": "cxhtest",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}
```

（2）输出

```json
{
    "cipherText": "encrypted data"
}
```

## 6. 数字信封解密

### 接口描述

- **接口名称**: /v1/DecByKeK
- **接口说明**: 使用数字信封解密数据
- **请求方式**: POST

### 输入参数

| **参数名称**        | **必选** | **类型** | **描述**                                           |
| ------------------- | -------- | -------- | :------------------------------------------------- |
| cipherText          | 是       | String   | 密文信息                                           |
| dataEnvelopAlgoType | 否       | String   | 默认为空，会根据pubkey信息查询解密使用的密码学算法 |
| kek                 | 是       | String   | 加密产生的KEK数据                                  |
| pubKey              | 是       | String   | 公钥，软件使用                                     |
| keyId               | 否       | String   | 硬件使用，和priKey二选一                           |

### 输出参数

| **参数名称** | **类型** | **描述**         |
| ------------ | -------- | ---------------- |
| plainText    | String   | 解密后的明文信息 |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/DecByKeK \
  -H 'Content-Type: application/json' \
  -d '{
    "cipherText": "87a9d1050019cfc5cbe9cbc86be0d2f0",
    "dataEnvelopAlgoType": "SM2SM4DataEnvelope",
    "kek": "0454c7ae043ef7b7cc07133fef3fc013d2eee934cce8be73d6a63150250834799f30995579c097ad08fc207439d9aa5bd8a77f699f078ea0279d0ff275b11bf91a278b3b4840e96e4720f630c7fe1787d4f8ec795ac6cdaac53292429f8bff9fe0ab4f3ce92564fdd64d55484eed4064e7",
    "keyId": "",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}'
```

### 示例

（1）输入

```json
{
    "cipherText": "87a9d1050019cfc5cbe9cbc86be0d2f0",
    "dataEnvelopAlgoType": "SM2SM4DataEnvelope",
    "kek": "0454c7ae043ef7b7cc07133fef3fc013d2eee934cce8be73d6a63150250834799f30995579c097ad08fc207439d9aa5bd8a77f699f078ea0279d0ff275b11bf91a278b3b4840e96e4720f630c7fe1787d4f8ec795ac6cdaac53292429f8bff9fe0ab4f3ce92564fdd64d55484eed4064e7",
    "keyId": "",
    "pubKey": "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2"
}
```

（2）输出

```json
{
    "plainText": "text example"
}
```

## 7. 数字信封使用公钥列表加密

### 接口描述

- **接口名称**: /v1/EncWithDeKWithPkList
- **接口说明**: 数字信封使用公钥列表加密数据
- **请求方式**: POST

### 输入参数

| **参数名称**        | **必选** | **类型**          | **描述**                                                     |
| ------------------- | -------- | ----------------- | :----------------------------------------------------------- |
| plainText           | 是       | String            | 明文信息                                                     |
| dataEnvelopAlgoType | 是       | String            | 加密使用的密码学算法,可选：SM2SM4DataEnvelope和Sm2CpAbeDataEnvelope(暂未实现) |
| pubKeyList          | 是       | array of [String] | 公钥列表                                                     |
| extraParam          | 否       | String            | 预留参数，暂未使用                                           |

### 输出参数

| **参数名称** | **类型**          | **描述** |
| ------------ | ----------------- | -------- |
| cipherText   | String            | 密文信息 |
| kekList      | array of [String] | kek列表  |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/EncWithDeKWithPkList \
  -H 'Content-Type: application/json' \
  -d '{
    "dataEnvelopAlgoType": "SM2SM4DataEnvelope",
    "plainText": "cxhtest",
    "pubKeyList": [ "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2", "040fd782e8cad8ce9563ea179156c7bc374f428b940c29e62a3a095dc5a2da4793480fe7998c45ba03bfb682e89aeff2c7b9b6703c45e85640b3434bf5ab21911c"
    ]
}'
```

### 示例

（1）输入

```json
{
    "dataEnvelopAlgoType": "SM2SM4DataEnvelope",
    "plainText": "cxhtest",
    "pubKeyList": [ "04e2b6224b3c3d261bd1b6cacd7cc48d8b673c36b26d43353918d6b7d3b716199f7bdb959be03b3499cd85343d1fd4e37523f1f738f63122abe6d976b990293eb2", "040fd782e8cad8ce9563ea179156c7bc374f428b940c29e62a3a095dc5a2da4793480fe7998c45ba03bfb682e89aeff2c7b9b6703c45e85640b3434bf5ab21911c"
    ]
}
```

（2）输出

```json
{
"cipherTextList": [
        "5601119022f339403861848512aaa451",
        "bf6e9ddec53fa76d1c53293513fcadce"
    ],
    "kekList": [ "04cb5823318d2c56b353fbddaf46f15d75a4e9acb4094fdd684f896df0e17df0e8ccbef15769490c916408967bef9bbdf001a1392e6b415493bf657f29a05179ecc7fbfee9866a65bdf45c2da60719b12c0a69ba64337f0d9b913b7cd601f365d43cc9809ae27b775d8fba6120b074a374", "04819642c369fc20f5903f88ad8d7de0c91c7a4b5faaea7b8b78f491a221d9276bb50509dd0c4132e6ceaf925a0805663c783aec2c97ba1207671f1b9ccd8737fc41d4ff517428c49ff717047b571873e220f49159255698c17b44acfefa77869b5636c8586c70d4be98f5e2d6b39d60e0"
    ]
}
```



## 8. 根据公钥获取私钥

### 接口描述

- **接口名称**: /v1/GetSkFromPk
- **接口说明**: 根据公钥获取私钥
- **请求方式**: POST

### 输入参数

| **参数名称** | **必选** | **类型** | **描述** |
| ------------ | -------- | -------- | -------- |
| pubKey       | 是       | String   | 公钥     |

### 输出参数

| **参数名称** | **类型** | **描述** |
| ------------ | -------- | -------- |
| sk           | String   | 私钥     |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/GetSkFromPk \
  -H 'Content-Type: application/json' \
  -d '{
	"pubKey": "2CC11CCAD50FCF9E12AD9C9239FE912C6F184C5A6E8F8CA2E0946545C0629C039389320AD5EF0F74FA2EDC1BFE6FF0314F0C902C88AA4F5CAD984630F068267D"
}'
```

### 示例

（1）输入

```json
{
	"pubKey": "2CC11CCAD50FCF9E12AD9C9239FE912C6F184C5A6E8F8CA2E0946545C0629C039389320AD5EF0F74FA2EDC1BFE6FF0314F0C902C88AA4F5CAD984630F068267D"
}
```

（2）输出

```json
{
    "code": 200,
    "msg": "success",
    "sk": "B6318F28F72B06C9885047370652E913C6134BA26A55311855BC5F9A2E5A11D4"
}
```

## 9. 根据 keyId 获取私钥

### 接口描述

- **接口名称**: /v1/GetSkFromKeyId
- **接口说明**: 根据 keyId 获取私钥
- **请求方式**: POST

### 输入参数

| **参数名称** | **必选** | **类型** | **描述**   |
| ------------ | -------- | -------- | ---------- |
| keyId        | 是       | String   | 密钥标识符 |

### 输出参数

| **参数名称** | **类型** | **描述** |
| ------------ | -------- | -------- |
| sk           | String   | 私钥     |

### 输入示例

```json
curl -X POST \
  http://127.0.0.1:8081/v1/GetSkFromKeyId \
  -H 'Content-Type: application/json' \
  -d '{
	"keyId": "20240511171121886"
}'
```

### 示例

（1）输入

```json
{
	"keyId": "20240511171121886"
}
```

（2）输出

```json
{
    "sk": "6717e550ec6a1a43d57e90c74d66743e"
}
```